@@ -1,5 +1,5 @@
 #
-# $Id: Headers.pm,v 1.31 1998/01/06 09:54:14 aas Exp $
+# $Id: Headers.pm,v 1.41 2001/04/12 06:50:28 gisle Exp $
 
 package HTTP::Headers;
 
@@ -29,8 +29,8 @@ The following methods are available:
 =cut
 
 use strict;
-use vars qw($VERSION);
-$VERSION = sprintf("%d.%02d", q$Revision: 1.31 $ =~ /(\d+)\.(\d+)/);
+use vars qw($VERSION $TRANSLATE_UNDERSCORE);
+$VERSION = sprintf("%d.%02d", q$Revision: 1.41 $ =~ /(\d+)\.(\d+)/);
 
 use Carp ();
 
@@ -69,10 +69,12 @@ my %header_order;
 my %standard_case;
 for (@header_order) {
     my $lc = lc $_;
-    $header_order{$lc} = $i++;
+    $header_order{$lc} = ++$i;
     $standard_case{$lc} = $_;
 }
 
+$TRANSLATE_UNDERSCORE = 1 unless defined $TRANSLATE_UNDERSCORE;
+
 
 
 =item $h = new HTTP::Headers
@@ -83,37 +85,37 @@ attribute-value pairs as parameters to the constructor.  I<E.g.>:
  $h = new HTTP::Headers
      Date         => 'Thu, 03 Feb 1994 00:00:00 GMT',
      Content_Type => 'text/html; version=3.2',
-     Content_Base => 'http://www.sn.no/';
+     Content_Base => 'http://www.perl.org/';
 
 =cut
 
 sub new
 {
     my($class) = shift;
-    my $self = bless {
-	'_header'   => { },
-    }, $class;
-
+    my $self = bless {}, $class;
     $self->header(@_); # set up initial headers
     $self;
 }
 
 
-=item $h->header($field [=> $val],...)
+=item $h->header($field [=> $value],...)
 
 Get or set the value of a header.  The header field name is not case
 sensitive.  To make the life easier for perl users who wants to avoid
 quoting before the => operator, you can use '_' as a synonym for '-'
-in header names.
+in header names (this behaviour can be suppressed by setting
+$HTTP::Headers::TRANSLATE_UNDERSCORE to a FALSE value).
 
-The value argument may be a scalar or a reference to a list of
-scalars. If the value argument is not defined, then the header is not
-modified.
+The header() method accepts multiple ($field => $value) pairs, so you
+can update several fields with a single invocation.
 
-The header() method accepts multiple ($field => $value) pairs.
+The optional $value argument may be a scalar or a reference to a list
+of scalars. If the $value argument is undefined or not given, then the
+header is not modified.
 
-The list of previous values for the last $field is returned.  Only the
-first header value is returned in scalar context.
+The old value of the last of the $field values is returned.
+Multi-valued fields will be concatenated with "," as separator in
+scalar context.
 
  $header->header(MIME_Version => '1.0',
 		 User_Agent   => 'My-Web-Client/0.01');
@@ -126,22 +128,71 @@ first header value is returned in scalar context.
 sub header
 {
     my $self = shift;
-    my($field, $val, @old);
-    while (($field, $val) = splice(@_, 0, 2)) {
+    my(@old);
+    while (my($field, $val) = splice(@_, 0, 2)) {
 	@old = $self->_header($field, $val);
     }
-    wantarray ? @old : $old[0];
+    return @old if wantarray;
+    return $old[0] if @old <= 1;
+    join(", ", @old);
 }
 
+
+=item $h->push_header($field, $val)
+
+Add a new field value of the specified header.  The header field name
+is not case sensitive.  The field need not already have a
+value. Previous values for the same field are retained.  The argument
+may be a scalar or a reference to a list of scalars.
+
+ $header->push_header(Accept => 'image/jpeg');
+
+=cut
+
+sub push_header
+{
+    Carp::croak('Usage: $h->push_header($field, $val)') if @_ != 3;
+    shift->_header(@_, 'PUSH');
+}
+
+=item $h->init_header($field, $val)
+
+Set the specified header to the given value unless it already has a
+value.  The header field name is not case sensitive.
+
+=cut
+
+sub init_header
+{
+    Carp::croak('Usage: $h->init_header($field, $val)') if @_ != 3;
+    shift->_header(@_, 'INIT');
+}
+
+
+=item $h->remove_header($field,...)
+
+This function removes the headers with the specified names.
+
+=cut
+
+sub remove_header
+{
+    my($self, @fields) = @_;
+    my $field;
+    foreach $field (@fields) {
+	$field =~ tr/_/-/ if $TRANSLATE_UNDERSCORE;
+	delete $self->{lc $field};
+    }
+}
+
+
 sub _header
 {
-    my($self, $field, $val, $push) = @_;
-    $field =~ tr/_/-/;  # allow use of '_' as alternative to '-' in fields
+    my($self, $field, $val, $op) = @_;
+    $field =~ tr/_/-/ if $TRANSLATE_UNDERSCORE;
 
     # $push is only used interally sub push_header
-
-    Carp::croak('Need a field name') unless defined $field;
-    Carp::croak('Too many parameters') if @_ > 4;
+    Carp::croak('Need a field name') unless length($field);
 
     my $lc_field = lc $field;
     unless(defined $standard_case{$lc_field}) {
@@ -150,23 +201,21 @@ sub _header
 	$standard_case{$lc_field} = $field;
     }
 
-    my $this_header = \@{$self->{'_header'}{$lc_field}};
+    my $h = $self->{$lc_field};
+    my @old = ref($h) ? @$h : (defined($h) ? ($h) : ());
 
-    my @old = ();
-    if (!$push && defined $this_header) {
-	@old = @$this_header;  # save it so we can return it
-    }
-    if (defined $val) {
-	@$this_header = () unless $push;
+    $op ||= "";
+    $val = undef if $op eq 'INIT' && @old;
+    if (defined($val)) {
+	my @new = ($op eq 'PUSH') ? @old : ();
 	if (!ref($val)) {
-	    # scalar: create list with single value
-	    push(@$this_header, $val);
+	    push(@new, $val);
 	} elsif (ref($val) eq 'ARRAY') {
-	    # list: copy list
-	    push(@$this_header, @$val);
+	    push(@new, @$val);
 	} else {
 	    Carp::croak("Unexpected field value $val");
 	}
+	$self->{$lc_field} = @new > 1 ? \@new : $new[0];
     }
     @old;
 }
@@ -176,13 +225,7 @@ sub _header
 # recommended "Good Practice" order.
 sub _header_cmp
 {
-    # Unknown headers are assign a large value so that they are
-    # sorted last.  This also helps avoiding a warning from -w
-    # about comparing undefined values.
-    $header_order{$a} = 999 unless defined $header_order{$a};
-    $header_order{$b} = 999 unless defined $header_order{$b};
-
-    $header_order{$a} <=> $header_order{$b} || $a cmp $b;
+    ($header_order{$a} || 999) <=> ($header_order{$b} || 999) || $a cmp $b;
 }
 
 
@@ -200,14 +243,17 @@ recommended "Good Practice" order.
 sub scan
 {
     my($self, $sub) = @_;
-    my $field;
-    foreach $field (sort _header_cmp keys %{$self->{'_header'}} ) {
-	my $list = $self->{'_header'}{$field};
-	if (defined $list) {
+    my $key;
+    foreach $key (sort _header_cmp keys %$self) {
+        next if $key =~ /^_/;
+	my $vals = $self->{$key};
+	if (ref($vals)) {
 	    my $val;
-	    for $val (@$list) {
-		&$sub($standard_case{$field} || $field, $val);
+	    for $val (@$vals) {
+		&$sub($standard_case{$key} || $key, $val);
 	    }
+	} else {
+	    &$sub($standard_case{$key} || $key, $vals);
 	}
     }
 }
@@ -249,31 +295,23 @@ sub as_string
 }
 
 
-# The remaining functions should autoloaded only when needed
-
-# A bug in 5.002gamma makes it risky to have POD text inside the
-# autoloaded section of the code, so we keep the documentation before
-# the __DATA__ token.
-
-=item $h->push_header($field, $val)
-
-Add a new field value of the specified header.  The header field name
-is not case sensitive.  The field need not already have a
-value. Previous values for the same field are retained.  The argument
-may be a scalar or a reference to a list of scalars.
-
- $header->push_header(Accept => 'image/jpeg');
-
-=item $h->remove_header($field,...)
-
-This function removes the headers with the specified names.
-
 =item $h->clone
 
 Returns a copy of this HTTP::Headers object.
 
 =back
 
+=cut
+
+sub clone
+{
+    my $self = shift;
+    my $clone = new HTTP::Headers;
+    $self->scan(sub { $clone->push_header(@_);} );
+    $clone;
+}
+
+
 =head1 CONVENIENCE METHODS
 
 The most frequently used headers can also be accessed through the
@@ -304,7 +342,7 @@ considered stale.
 =item $h->if_unmodified_since
 
 This header is used to make a request conditional.  If the requested
-resource has not been modified since the time specified in this field,
+resource has (not) been modified since the time specified in this field,
 then the server will return a C<"304 Not Modified"> response instead of
 the document itself.
 
@@ -376,7 +414,7 @@ This header should contain an Internet e-mail address for the human
 user who controls the requesting user agent.  The address should be
 machine-usable, as defined by RFC822.  E.g.:
 
-  $h->from('Gisle Aas <aas@sn.no>');
+  $h->from('King Kong <king@kong.com>');
 
 =item $h->referer
 
@@ -421,46 +459,8 @@ header instead.
 
 =back
 
-=head1 COPYRIGHT
-
-Copyright 1995-1997 Gisle Aas.
-
-This library is free software; you can redistribute it and/or
-modify it under the same terms as Perl itself.
-
 =cut
 
-1;
-
-#__DATA__
-
-sub clone
-{
-    my $self = shift;
-    my $clone = new HTTP::Headers;
-    $self->scan(sub { $clone->push_header(@_);} );
-    $clone;
-}
-
-sub push_header
-{
-    Carp::croak('Usage: $h->push_header($field, $val)') if @_ != 3;
-    shift->_header(@_, 'PUSH');
-}
-
-
-sub remove_header
-{
-    my($self, @fields) = @_;
-    my $field;
-    foreach $field (@fields) {
-	$field =~ tr/_/-/;
-	delete $self->{'_header'}{lc $field};
-    }
-}
-
-# Convenience access functions
-
 sub _date_header
 {
     require HTTP::Date;
@@ -491,7 +491,7 @@ sub client_date         { shift->_date_header('Client-Date',         @_); }
 
 sub content_type      {
   my $ct = (shift->_header('Content-Type', @_))[0];
-  return '' unless defined $ct;
+  return '' unless defined($ct) && length($ct);
   my @ct = split(/\s*;\s*/, lc($ct));
   wantarray ? @ct : $ct[0];
 }
@@ -506,8 +506,8 @@ sub server            { (shift->_header('Server',           @_))[0] }
 
 sub from              { (shift->_header('From',             @_))[0] }
 sub referer           { (shift->_header('Referer',          @_))[0] }
-sub etag              { (shift->_header('ETag',             @_))[0] }
 sub warning           { (shift->_header('Warning',          @_))[0] }
+*referrer = \&referer;  # on tchrist's request
 
 sub www_authenticate  { (shift->_header('WWW-Authenticate', @_))[0] }
 sub authorization     { (shift->_header('Authorization',    @_))[0] }
@@ -534,7 +534,16 @@ sub _basic_auth {
 	return $val unless wantarray;
 	return split(/:/, $val, 2);
     }
-    undef;
+    return;
 }
 
+=head1 COPYRIGHT
+
+Copyright 1995-2001 Gisle Aas.
+
+This library is free software; you can redistribute it and/or
+modify it under the same terms as Perl itself.
+
+=cut
+
 1;
